var isBannerAdShowing = false;
var hasLocalNotificationsPermission = false;
var logs = '';
var title = '';
var price = '';
var quantity = '';

var app = {
    // Application Constructor
    initialize: function() {
        app.bindEvents();
    },

    bindEvents: function() {
        document.addEventListener('deviceready', app.onDeviceReady);

        // Assign callbacks to the demo buttons
        btn_interstitial.addEventListener('touchstart', app.onShowInterstitialAdClicked);
        btn_rewarded.addEventListener('touchstart', app.onShowRewardedAdClicked);
        btn_banner.addEventListener('touchstart', app.onToggleBannerAdClicked);
        btn_offerwall.addEventListener('touchstart', app.onShowOfferwallClicked);
        btn_specialoffer.addEventListener('touchstart', app.onShowSpecialOfferClicked);
        btn_analytics.addEventListener('touchstart', app.onLogEventClicked);
        btn_enableNotification.addEventListener('touchstart', app.onEnableLocalNotificationClicked);
        btn_disableNotification.addEventListener('touchstart', app.onDisableLocalNotificationClicked);
        btn_purchase.addEventListener('touchstart', app.onPurchaseClicked);
        btn_consume.addEventListener('touchstart', app.onConsumeClicked);
        btn_gdprOptIn.addEventListener('touchstart', app.onGDPROptInClicked);
        btn_gdprOptOut.addEventListener('touchstart', app.onGDPROptOutClicked);
        btn_gdprShowDialogs.addEventListener('touchstart', app.onGDPRShowDialogsClicked);
    },

    onDeviceReady: function() {
        // Ask for permission to show local notifications
        // This can be done at any point in the application
        
        var onPermissionGranted = function() {
            hasLocalNotificationsPermission = true;
        };

        var onPermissionRefused = function() {
            hasLocalNotificationsPermission = false;
        };

        Enhance.requestLocalNotificationPermission(onPermissionGranted, onPermissionRefused);

        // Set currency callback for offerwalls
        // It should be done as soon as possible

        Enhance.setReceivedCurrencyCallback(app.onCurrencyReceived);
    },

    // Interstitial Ad

    onShowInterstitialAdClicked: function() {
    	var isReadyCallback = function(result) {
    		// Check whether an interstitial ad is available

    		if(!result) {
    			app.writeLog('Interstitial ad is not ready');
    			return;
    		}

            // Show the ad

            Enhance.showInterstitialAd();
            app.writeLog('Showing interstitial ad');
    	};

        Enhance.isInterstitialReady(isReadyCallback);
    },
    
    // Rewarded Ad

    onShowRewardedAdClicked: function() {
    	var isReadyCallback = function(result) {
    		// Check whether a rewarded ad is available

    		if(!result) {
    			app.writeLog('Rewarded ad is not ready');
    			return;
    		}

    		// It's ready

            var onRewardGranted = function(rewardValue, rewardType) {
                // Reward is granted (user watched the ad for example)
                // You can check reward type:
                
                if(rewardType == Enhance.RewardType.ITEM) 
                    app.writeLog("Reward granted (item)");
                else if(rewardType == Enhance.RewardType.COINS)
                    app.writeLog("Reward granted (coins), value: " + rewardValue);
                else
                    app.writeLog("Unknown reward granted");
            };

            var onRewardDeclined = function() {
                // Reward is declined (user closed the ad for example)
                app.writeLog('Reward declined');
            };

            var onRewardUnavailable = function() {
                // Reward is unavailable (network error for example)
                app.writeLog('Reward unavailable');
            };

    		Enhance.showRewardedAd(onRewardGranted, onRewardDeclined, onRewardUnavailable);
            app.writeLog('Showing rewarded ad');
    	};

    	Enhance.isRewardedAdReady(isReadyCallback);
    },

    // Banner Ad

    onToggleBannerAdClicked: function() {
    	// Check if banner ad is already displayed, then hide it:

    	if(isBannerAdShowing) {
    		isBannerAdShowing = false;
    		Enhance.hideBannerAd();
    		app.writeLog('Hiding banner ad');
    		return;
    	}

    	// It's not displayed
    	// Check if any banner ad is available and show it if possible

    	var isReadyCallback = function(result) {
    		if(!result) {
    			app.writeLog('Banner ad is not ready');
    			return;
    		}

    		// It's available
    		// Display ad in the bottom of the screen
    		// You can also change the position to Enhance.Position.TOP

    		isBannerAdShowing = true;
    		Enhance.showBannerAdWithPosition(Enhance.Position.BOTTOM);
    		app.writeLog('Showing banner ad');
    	};

    	Enhance.isBannerAdReady(isReadyCallback);
    },

    // Offerwall

    onShowOfferwallClicked: function() {
    	// Check whether an offerwall is available

       var isReadyCallback = function(result) {
       		if(!result) {
       			app.writeLog('Offerwall is not ready');
       			return;
       		}	

       		// It's ready, display the offerwall

       		Enhance.showOfferwall();
       		app.writeLog('Showing offerwall');
       };

       Enhance.isOfferwallReady(isReadyCallback);
    },

    // Offerwall callback

    onCurrencyReceived: function(amount) {
        app.writeLog('Currency received: ' + amount);
    },

    // Special Offer

    onShowSpecialOfferClicked: function() {
    	// Check whether a special offer is available

    	var isReadyCallback = function(result) {
       		if(!result) {
       			app.writeLog('Special offer is not ready');
       			return;
       		}	

       		// It's ready, display the special offer

       		Enhance.showSpecialOffer();
       		app.writeLog('Showing special offer');
       };

       Enhance.isSpecialOfferReady(isReadyCallback);
    },

    // Analytics

    onLogEventClicked: function() {
    	// Send an event to analytics network(e.g. Google Analytics)

        // Simple event:
        Enhance.logEvent('event_type');
        // Event with an additional parameter:
        Enhance.logEvent('event_type', 'event_param_key', 'event_param_value');

        app.writeLog('Sent analytics event');
    },

    // Local Notification

    onEnableLocalNotificationClicked: function() {
    	// Check whether you have a permission to schedule local notifications

    	if(!hasLocalNotificationsPermission) {
    		app.writeLog('Permission for local notifications is not granted');
    		return;
    	}

    	// We have permission, enable a new local notification
    	// It will appear 5 seconds after the app is deactivated

    	Enhance.enableLocalNotification('Enhance', 'Local Notification!', 5);
    	app.writeLog('Enabled local notification');
    },

    onDisableLocalNotificationClicked: function() {
    	// Cancel previously scheduled local notification

    	Enhance.disableLocalNotification();
    	app.writeLog('Disabled local notification');
    },

    // In-App Purchases

    onPurchaseClicked: function() {
        Enhance.purchases.attemptPurchase('Enhance_SKU_One', function() {
            // success
            app.updateIAPLabel();
            app.writeLog('Purchase success');
        }, function() {
            // failed
            app.writeLog('Purchase failed');
        });
    },

    onConsumeClicked: function() {
        Enhance.purchases.consume('Enhance_SKU_One', function() {
            // success
            app.updateIAPLabel();
            app.writeLog('Consume success');
        }, function() {
            // failed
            app.writeLog('Consume failed');
        });
    },

    updateIAPLabel: function() {
        Enhance.purchases.getDisplayTitle('Enhance_SKU_One', 'Default Title', function(result) {
            title = result;
            app.onIAPLabelUpdated();
        });

        Enhance.purchases.getDisplayPrice('Enhance_SKU_One', 'Default Price', function(result) {
            price = result;
            app.onIAPLabelUpdated();
        });

        Enhance.purchases.getOwnedItemCount('Enhance_SKU_One', function(result) {
            quantity = result;
            app.onIAPLabelUpdated();
        });
    },

    onIAPLabelUpdated: function() {
        label.innerHTML = title + ' - ' + price + ' - Qty: ' + quantity;
    },

    // GDPR

    onGDPROptInClicked: function() {
        Enhance.serviceTermsOptIn();
        app.writeLog("Called explicit opt-in");
    },

    onGDPROptOutClicked: function() {
        Enhance.serviceTermsOptOut();
        app.writeLog("Called explicit opt-out");
    },

    onGDPRShowDialogsClicked: function() {
        Enhance.requiresDataConsentOptIn(function(result) {
            if(result) {
                Enhance.showServiceOptInDialogs(null, function() {
                    app.writeLog("Finished displaying opt-in dialogs");
                });
            }
        });
    },

    // Write logs in the textarea

    writeLog: function(text) {
        logs += text + '\n';
        log_output.value = logs;

        // Keep textArea scrolled to the bottom
        log_output.scrollTop = log_output.scrollHeight;

        console.log(text);
    }
};

app.initialize();
